internal

package
v0.1.1 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Dec 15, 2025 License: MIT Imports: 2 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

This section is empty.

Types

type StackPool

type StackPool struct {
	// contains filtered or unexported fields
}

StackPool is used for pooling the scoped stacks.

func (*StackPool) CheckIn

func (this *StackPool) CheckIn(value []contracts.ScopedInstance)

CheckIn returns the scoped stack back to this pool.

func (*StackPool) CheckOut

func (this *StackPool) CheckOut() []contracts.ScopedInstance

CheckOut will find or generate a new scoped stack and return it.

Directories

Path Synopsis

Jump to