Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type StackPool ¶
type StackPool struct {
// contains filtered or unexported fields
}
StackPool is used for pooling the scoped stacks.
func (*StackPool) CheckIn ¶
func (this *StackPool) CheckIn(value []contracts.ScopedInstance)
CheckIn returns the scoped stack back to this pool.
func (*StackPool) CheckOut ¶
func (this *StackPool) CheckOut() []contracts.ScopedInstance
CheckOut will find or generate a new scoped stack and return it.
Click to show internal directories.
Click to hide internal directories.